
setwd("path")

library(e1071)
library(lqmm)
library(lme4qtl)
library(kinship2)

#Loading Data
data<-read.table("data_sardinian.csv", header=T, sep=";")

#Calculating kinship matrix
data<-data[order(data$SUBFAMID),]
kped<-with(data, pedigree(id=data$id, dadid=data$father_id, momid=data$mother_id, sex=data$sex, famid=data$FAMID))
kmat <- kinship(kped)
kmat<-2*kmat
kmat<-as.matrix(kmat);


#Excluding subjects without protein levels and without age at blood sampling
data$ms<-as.character(data$ms)
data$sex<-as.character(data$sex)
data<-data[which(!is.na(data$arg2_hpa000663)),]
data<-data[which(!is.na(data$Age)),]

#Building kinship random effect martrix, using Cholesky decomposition, to be used in LQMM model
data<-data[order(data$SUBFAMID),]
km<-chol(kmat[as.character(data$id), as.character(data$id)])
ccc<-rep(1,206)
Zt2 <- km %*% ccc

#Protein of interest
name<-"c9_hpa029577"

  data<-data[order(data$id),]
  
  #Standardizing protein levels using mean and SD in healthy controls
  sd<-sd(data[data$ms==0,as.character(name)])
  meancontr<-mean(data[data$ms==0,as.character(name)])
  data[,as.character(name)]<-(data[,as.character(name)]-meancontr)/sd
  
  #Model
  model <- as.formula(paste(name, " ~ ms+Age+sex+ (1|id) + (1|SUBFAMID)", collapse = " "))

  kmat2<-kmat[as.character(data$id), as.character(data$id)]
  
  #LMM
  m1 <- relmatLmer(model, data=data,REML=F, relmat = list(id = kmat2))
  
  #Checking residuals using Kim guidelines (https://www.ncbi.nlm.nih.gov/pmc/articles/PMC3591587/)
  residuals<-m1@resp[["wtres"]]
  N<-nrow(data)
  Vskew<- 6*N*(N-1) / ((N-2)*(N+1)*(N+3))
  Vkur<- 4*(N^2-1)*Vskew / ((N-3)*(N+5))
  skew2<-as.numeric(skewness(residuals))/sqrt(Vskew)
  kurt2<-as.numeric(kurtosis(residuals))/sqrt(Vkur)
  
  #1 if normality of residuls is respected, else 0
  norm<-ifelse(abs(as.numeric(skew2)) > 3.29 | abs(as.numeric(kurt2)) > 3.29, 0,1)
  
  if(norm==1){
    #LMM model
    m1<-summary(m1)
   estimate<-m1$coefficients[2,1]
   se<-m1$coefficients[2,2]
  pval<-signif(1 - pchisq((m1$coefficients[2,1]/m1$coefficients[2,2])^2, 1), 7)
  LCL<-estimate - qnorm(0.975)*se
  UCL<-estimate + qnorm(0.975)*se
  }
  
  if(norm==0){
    #LQMM model

    data<-data[order(data$SUBFAMID),]
    
    model <- as.formula(paste(name, " ~ ms + Age + sex", collapse = " "))
    
    rqr<-lqmm(fixed=model, random = ~ Zt2, cov="pdDiag", group=SUBFAMID, 
              type="robust", data=data, na.action=na.omit, tau=c(0.5),  control = lqmmControl(method = "gs", LP_max_iter=5000))
    
    #Bootstrap
    fit.boot <- lqmm::boot(rqr, R = 1000, seed = 24061994, startQR = FALSE)
    summary(fit.boot)
    B <- extractBoot(fit.boot, "fixed")[, "ms1"]
    
    
    estimate<-rqr$theta[2]
    se<-sd(B, na.rm=T)
    meanB<-mean(B, na.rm=T)
    bias<-estimate-meanB
    biasthresh<-0.25*se
    pval<-2*pt(-abs((estimate)/se), (nrow(data)-rqr$df - 1))
    LCL<-estimate+qt(0.05/2, (nrow(data) -rqr$df - 1))*se
    UCL<-estimate+qt(1-0.05/2, (nrow(data)-rqr$df - 1))*se
    
    
    #Bias-corrected p-value
    if(abs(bias)>biasthresh){
      pval<-2*pt(-abs((2*estimate-meanB)/se), (nrow(data)-rqr$df - 1))
      LCL<-2*estimate-meanB+qt(0.05/2, (nrow(data)-rqr$df - 1))*se
      UCL<-2*estimate-meanB+qt(1-0.05/2, (nrow(data)-rqr$df - 1))*se
    }

  }
    

