setwd("path")

library(lme4qtl)
library(kinship2)

#Loading Data
data<-read.table("data_sardinian.csv", header=T, sep=";")

#Calculating kinship matrix
data<-data[order(data$SUBFAMID),]
kped<-with(data, pedigree(id=data$id, dadid=data$father_id, momid=data$mother_id, sex=data$sex, famid=data$SUBFAMID))
kmat <- kinship(kped)
kmat<-2*kmat
kmat<-as.matrix(kmat);


#Excluding subjects without protein levels and without age at blood sampling
data<-data[which(!is.na(data$arg2_hpa000663)),]
data<-data[which(!is.na(data$Age)),]

#Protein and SNP of interest
name<-"c9_hpa029577"
namesnp<-"imm_10_6139051"

#Standardizing protein levels
data_controls<-data[data$ms==0 & !is.na(data$imm_10_6139051),]
sd<-sd(data_controls[,as.character(name)])
meancontr<-mean(data_controls[,as.character(name)])
data_controls[,as.character(name)]<-(data_controls[,as.character(name)]-meancontr)/sd

#LMM model
model <- as.formula(paste(name, " ~ (1|id) + (1|SUBFAMID) + Age+ sex+", namesnp, collapse = " "))

kmat2<-kmat[as.character(data_controls$id), as.character(data_controls$id)]
m1 <- relmatLmer(model, data=data_controls,REML=F, relmat = list(id = kmat2))
m1<-summary(m1)
estimate<-m1$coefficients[4,1]
se<-m1$coefficients[4,2]
pval<-signif(1 - pchisq((m1$coefficients[4,1]/m1$coefficients[4,2])^2, 1), 7)
LCL<-estimate - qnorm(0.975)*se
UCL<-estimate + qnorm(0.975)*se